# -*- coding: utf-8 -*-
import scrapy
from scrapy_redis.spiders import RedisSpider
from zbpblog.items import ZbpblogItem
from time import time

class ZbpblogSpider(RedisSpider):   # 原scrapy继承scrapy.Spider蜘蛛，现在则继承scrapy_redis的RedisSpider蜘蛛
    name = 'Zbpblog'
    redis_key = "zbpblog:start_urls"        # 声明用于存储起始url的redis列表的key

    def __init__(self, *args, **kwargs):
        # 动态定义 allowed_domains 成员
        domains = kwargs.pop('domain', '')
        self.allowed_domains = list(map(lambda x: x.strip(" "), domains.split(",")))

        # 别忘了调用父类的构造函数
        super(ZbpblogSpider, self).__init__(*args, **kwargs)

    def parse(self, response):
        # 先解析首页中的栏目页url
        cate_url_path = "//ul[@id='header-cate-ul']/a/@href"
        yield from self.__yieldRequestByXpath(response=response, xpath=cate_url_path, callback=self.parseDetailUrl)

        # 再解析首页的详情页url
        yield from self.parseDetailUrl(response)  # 将parseDetailUrl产出的Request产出给parse的调用方（yield from相当于是一个通道，将子生成器parseDetailUrl的产出内容发送给parse的调用方）

    # 解析详情页内容
    def parseDetail(self, response):
        item = ZbpblogItem()
        item["title"] = response.xpath("//div[@id='blog-art-head']/h1/text()").extract_first()
        item["category"] = response.xpath("//div[@id='blog-art-info']/span[1]/a/text()").extract_first()
        item["url"] = response.url
        item["create_time"] = int(time())
        item["content"] = response.xpath("//div[@class='blog-art-cont']/div[@class='cont']").extract_first()

        yield dict(item)

    # 解析详情页url
    def parseDetailUrl(self, response):
        # 请求栏目页的分页
        page_url_xpath = "//div[@id='page']//a/@href"
        yield from self.__yieldRequestByXpath(response=response, xpath=page_url_xpath, callback=self.parseDetailUrl)

        # 请求栏目页中的详情页
        detail_url_xpath = "//div[contains(@class,'blog-info')]//h2[@class='blog-h2']/a/@href"
        yield from self.__yieldRequestByXpath(response=response, xpath=detail_url_xpath, callback=self.parseDetail)

    def __yieldRequestByXpath(self, response, xpath, callback):
        urls = response.xpath(xpath).extract()
        for url in urls:
            complete_url = response.urljoin(url)
            yield scrapy.Request(complete_url, callback=callback)